use std::{
    collections::HashMap,
    sync::Arc,
    time::{Duration, Instant},
};

use minisign::SecretKey;
use tokio::sync::Mutex;
use tokio_util::sync::CancellationToken;
use tracing::debug;

#[derive(Debug, Clone)]
pub struct PrivateKeyCache {
    cache: Arc<Mutex<HashMap<String, PrivateKeyEntry>>>,
}

#[derive(Debug, Clone)]
pub struct PrivateKeyEntry {
    key: SecretKey,
    ttl: Duration,
    creation: Instant,
    clean_token: CancellationToken,
}

impl PrivateKeyEntry {
    pub fn secret_key(&self) -> SecretKey {
        self.key.clone()
    }
}

impl PrivateKeyCache {
    pub fn new() -> Self {
        Self {
            cache: Arc::new(Mutex::new(HashMap::new())),
        }
    }

    pub async fn add_key(&self, user: &str, key: &SecretKey, ttl: Duration) {
        // create token to abort cleaning task
        let clean_token = CancellationToken::new();
        // create private key entry
        let entry = PrivateKeyEntry {
            key: key.clone(),
            ttl,
            creation: Instant::now(),
            clean_token: clean_token.clone(),
        };
        // lock cache
        let mut cache = self.cache.lock().await;
        // add private key to cache
        if let Some(old_entry) = cache.insert(user.to_string(), entry) {
            // abort cleaning task of old entry
            old_entry.clean_token.cancel();
        }

        // spawn task to remove private key from cache after storage expiration
        let cache = self.cache.clone();
        let user = user.to_string();
        tokio::spawn(async move {
            tokio::select! {
                _ = tokio::time::sleep(ttl) => {
                    let mut lock = cache.lock().await;
                    lock.remove(&user);
                    debug!("Deleted cached Secret Key after TTL expiration")
                },
                _ = clean_token.cancelled() => {
                    debug!("Cancelled automated cache clearance after TTL renewal")
                }
            }
        });
    }

    pub async fn get_key(&self, user_id: &str) -> Option<PrivateKeyEntry> {
        let lock = self.cache.lock().await;
        lock.get(user_id).cloned()
    }
}
