use std::{future::IntoFuture, net::Ipv4Addr, time::Duration};

use axum::http::HeaderName;
use axum_jwt_login::{AuthSessionWithApiKeyBuilder, JWTKeyPair};
use backend::ApiBackend;
use error_stack::{Report, ResultExt};
use routes::api_keys::models::ApiKey;
use sqlx::PgPool;
use tokio::net::TcpListener;
use tokio_util::sync::CancellationToken;
use tracing::info;

use crate::APP_NAME;
use crate::{config::Configuration, errors::AppError};

pub mod backend;
mod description;
pub mod routes;

pub async fn start(
    config: &Configuration,
    cancellation_token: CancellationToken,
) -> Result<(), Report<AppError>> {
    // Initialize database pool
    let pool = PgPool::connect_lazy(&config.database_url).change_context(AppError)?;
    // Get API Keys from database
    let api_keys = ApiKey::get_all_with_secret_attached(&pool, config)
        .await
        .change_context(AppError)?;

    // Initialize API Backend
    let backend = ApiBackend::new(pool.clone(), config);
    // Create Session storage for API
    let session = AuthSessionWithApiKeyBuilder::new()
        .backend(backend)
        .api_header_name(HeaderName::from_static("x-api-key"))
        .initial_api_keys(api_keys)
        .jwt_key_pair(JWTKeyPair::from_secret(&config.token_secret))
        .session_length(Duration::from_secs(config.session_length))
        .build()
        .change_context(AppError)?;

    // Create routes
    let router = routes::create_routes(session);

    // Define listener
    let port = config.port;
    let listener = TcpListener::bind((Ipv4Addr::LOCALHOST, port))
        .await
        .change_context(AppError)?;

    // Serve API
    let server = axum::serve(listener, router.into_make_service())
        .with_graceful_shutdown(shutdown_signal(cancellation_token));

    tokio::spawn(server.into_future());

    info!("{APP_NAME} up and running on port '{port}'.");

    Ok(())
}

async fn shutdown_signal(stop_signal: CancellationToken) {
    // Wait for shutdown event.
    stop_signal.cancelled().await;

    info!("Shutting down {APP_NAME}...");
}
