use axum::{debug_handler, Json};

use crate::{
    api::{
        description::API_KEY_TAG,
        routes::{
            api_keys::{models::ApiKey, sql},
            AuthBackendType,
        },
    },
    errors::ApiError,
};

#[debug_handler]
#[utoipa::path(
    post,
    path = "/apikeys",
    summary = "Create new API Key",
    description = "Create a new API Key.",
    request_body(content = ApiKey, description = "API Key details", content_type = "application/json"),
    responses(
        (status = OK, description = "API Key successfully created (API Key Secret in Body)", body = String),
    ),
    security(
        ("user_auth" = ["write:apikeys",]),
    ),
    tag = API_KEY_TAG)]
pub async fn create_api_key(
    auth_session: AuthBackendType,
    Json(api_key): Json<ApiKey>,
) -> Result<String, ApiError> {
    let backend = auth_session.backend();

    // create new API key
    let (key_secret, key) = ApiKey::create(
        &api_key.name,
        api_key.auth_required,
        api_key.permissions.0,
        backend.config(),
    )?;

    // insert API Key into database
    sql::create_api_key(backend.pool(), &key).await?;

    // add API Key to session
    auth_session.add_api_key(key).await;

    // Return key secret in response
    Ok(key_secret)
}
