use std::collections::HashSet;

use axum_jwt_login::ApiKey as ApiKeyTrait;
use blake3::{Hash, Hasher};
use chrono::NaiveDateTime;
use serde::{Deserialize, Serialize};
use sqlx::PgPool;
use ts_rs::TS;
use utoipa::ToSchema;
use uuid::Uuid;

use crate::{
    api::routes::{
        models::Status,
        users::permissions::{Permission, PermissionContainer},
    },
    config::Configuration,
    errors::ApiError,
    utils::create_random,
};

const KEY_LENGTH: usize = 40;

#[derive(Debug, Clone, Serialize, Deserialize, TS, ToSchema)]
#[ts(export)]
pub struct ApiKey {
    pub id: String,
    #[serde(skip)] // Don't leak Hash
    pub hash: String,
    pub name: String,
    pub auth_required: bool,
    #[schema(inline)]
    pub permissions: PermissionContainer,
    #[serde(skip)] // Don't leak secret
    pub api_config_secret: Option<String>,
    #[serde(default)]
    #[schema(value_type = String, read_only)]
    pub creation_date: Option<NaiveDateTime>,
    #[serde(default)]
    #[schema(value_type = String, read_only)]
    pub last_change: Option<NaiveDateTime>,
    pub status: Status,
}

impl ApiKey {
    pub async fn get_all_with_secret_attached(
        pool: &PgPool,
        config: &Configuration,
    ) -> Result<Vec<Self>, ApiError> {
        let mut keys = super::sql::get_api_keys(pool).await?;

        for key in keys.iter_mut() {
            key.api_config_secret = Some(config.token_secret.clone());
        }

        Ok(keys)
    }

    pub fn create(
        name: &str,
        requires_auth: bool,
        permissions: HashSet<Permission>,
        config: &Configuration,
    ) -> Result<(String, Self), ApiError> {
        // create uuid
        let uuid = Uuid::new_v4().simple();
        // create API Key secret part
        let key: String = create_random(KEY_LENGTH);
        // calculate API Key signature
        let mut hasher = Hasher::new_derive_key(&config.token_secret);
        hasher.update(key.as_bytes());
        let signature = hasher.finalize();

        // Return Api Key Secret and Api Key
        Ok((
            key,
            Self {
                name: name.to_string(),
                auth_required: requires_auth,
                id: uuid.to_string(),
                hash: signature.to_hex().to_string(),
                permissions: PermissionContainer(permissions),
                api_config_secret: Some(config.token_secret.clone()),
                creation_date: None,
                last_change: None,
                status: Status::Active,
            },
        ))
    }

    pub fn validate(&self, key_secret_part: &str) -> Result<(), ApiError> {
        // calculate API Key signature
        // let mut mac = Hmac::<Sha512>::new_from_slice(
        //     self.api_config_secret
        //         .clone()
        //         .ok_or(ApiError::InternalError(
        //             "Missing API Config Secret".to_string(),
        //         ))?
        //         .as_bytes(),
        // )?;
        // mac.update(key_secret_part.as_bytes());
        let mut hasher = Hasher::new_derive_key(&self.api_config_secret.clone().ok_or(
            ApiError::InternalError("Missing API Config Secret".to_string()),
        )?);
        hasher.update(key_secret_part.as_bytes());
        let signature = hasher.finalize();

        match signature == Hash::from_hex(self.hash.clone())? {
            true => Ok(()),
            false => Err(ApiError::AccessDenied),
        }
        // match mac.verify_slice(&self.hash) {
        //     Ok(_) => Ok(()),
        //     Err(_e) => Err(ApiError::AccessDenied),
        // }
    }
}

impl ApiKeyTrait for ApiKey {
    type Permission = Permission;

    fn id(&self) -> String {
        self.id.clone()
    }
    fn validate(&self, key_secret_part: &str) -> bool {
        self.validate(key_secret_part).is_ok()
    }
    fn requires_user_auth(&self) -> bool {
        self.auth_required
    }
    fn get_permissions(&self) -> HashSet<Self::Permission> {
        self.permissions.0.clone()
    }
}

#[cfg(test)]
mod test {
    use std::collections::HashSet;

    use crate::config::Configuration;

    use super::ApiKey;

    #[tokio::test]
    async fn test_api_key_creation_and_validation() {
        let config = Configuration {
            token_secret: "abcdefghijk".to_string(),
            ..Default::default()
        };
        let (key, api_key) =
            ApiKey::create("name", true, HashSet::new(), &config).expect("Error creating API Key");
        println!("{key}, {api_key:?}");
        assert_eq!(Ok(()), api_key.validate(&key));
        assert_ne!(Ok(()), api_key.validate("124"));
    }
}
