use std::sync::Arc;

use axum::{
    debug_handler,
    response::{
        sse::{Event, KeepAlive},
        Sse,
    },
    Extension,
};
use futures::stream::Stream;

use tokio_stream::{wrappers::BroadcastStream, StreamExt as _};

use crate::{
    api::{
        description::DATA_HUB_TAG,
        routes::{
            data_hub::models::{DataHub, DataHubStream},
            AuthBackendType,
        },
    },
    errors::ApiError,
};

#[debug_handler]
#[utoipa::path(
    get,
    path = "/dataHub",
    summary = "Subscribe to real-time data updates",
    description = "This endpoint streams real-time data updates to the client using server-sent events (SSE).
       The client must establish a persistent HTTP connection to receive updates.",
    responses(
        (status = OK, body = DataHubStream, description = "Stream of real-time data updates",content_type = "text/event-stream"),
    ),
     tag = DATA_HUB_TAG)]
pub async fn sse_data_hub(
    _auth_session: AuthBackendType,
    Extension(data_hub): Extension<Arc<DataHub>>,
) -> Sse<impl Stream<Item = Result<Event, ApiError>>> {
    println!("{data_hub:?}");
    // use BroadcastStream to convert Receiver into Stream
    // alternatively use async-stream (https://github.com/tokio-rs/axum/discussions/1060)
    let rx = data_hub.receiver.resubscribe();

    // convert stream values into SSE events
    let stream = BroadcastStream::from(rx).map(|value| match value {
        Ok(value) => Ok(value.into()),
        Err(_) => Ok(Event::default()), // send event where no subscription is on,
    });

    Sse::new(stream).keep_alive(KeepAlive::default())
}
