use serde::{Deserialize, Serialize};
use sqlx::types::JsonValue;
use tracing::error;
use ts_rs::TS;
use utoipa::ToSchema;
use validator::{Validate, ValidationError};

use crate::api::routes::{models::Status, users::models::ShortUser};

#[derive(Debug, Clone, Serialize, Deserialize, TS, ToSchema, Validate)]
#[ts(export)]
pub struct Department {
    pub id: i32,
    pub name: String,
    pub description: Option<String>,
    #[schema(inline)]
    #[validate(custom(function = "validate_manager", message = "Missing Manager"))]
    pub manager: Manager,
    #[schema(inline)]
    #[validate(custom(function = "validate_manager", message = "Missing Vice Manager"))]
    pub vice_manager: Manager,
    pub status: Status,
}

#[derive(Debug, Clone, Serialize, Deserialize, TS, ToSchema)]
pub struct Manager(pub Option<ShortUser>);

impl From<Option<JsonValue>> for Manager {
    fn from(value: Option<JsonValue>) -> Self {
        Self(
            value
                .map(|value| -> Option<ShortUser> {
                    match serde_json::from_value::<ShortUser>(value) {
                        Ok(manager) => Some(manager),
                        Err(err) => {
                            error!("{err:?}");
                            None
                        }
                    }
                })
                .flatten(),
        )
    }
}

fn validate_manager(value: &Manager) -> Result<(), ValidationError> {
    match value.0 {
        Some(_) => Ok(()),
        None => Err(ValidationError::new("Missing Manager")),
    }
}

#[derive(Debug, Clone, Serialize, Deserialize, TS, ToSchema)]
#[ts(export)]
pub struct ShortDepartment {
    pub id: i32,
    pub name: String,
    pub description: Option<String>,
}
