use sqlx::PgPool;

use crate::{api::routes::models::Status, errors::ApiError};

use super::models::Department;

pub async fn get_departments(
    pool: &PgPool,
    filter_active: Option<Status>,
) -> Result<Vec<Department>, ApiError> {
    Ok(sqlx::query_as!(
        Department,
        r#"SELECT
            DEPARTMENTS."DepartmentID" as id,
            DEPARTMENTS."DepartmentName" as name,
            DEPARTMENTS."Description" as description,
            DEPARTMENTS."Status" as "status: Status",
            json_build_object(
                'user_id', MANAGER."UserID", 
                'name', MANAGER."Name",
                'surname', MANAGER."Surname",
                'email', MANAGER."Email"
            ) AS manager,
            json_build_object(
                'user_id', vManager."UserID", 
                'name', vManager."Name",
                'surname', vManager."Surname",
                'email', vManager."Email"
            ) AS vice_manager
        FROM
            departments
        LEFT JOIN 
            USERS MANAGER ON DEPARTMENTS."ManagerID" = MANAGER."UserID"
        LEFT JOIN
            USERS vManager ON DEPARTMENTS."ViceManagerID" = vManager."UserID"
        WHERE
            ($1::smallint IS NULL OR DEPARTMENTS."Status" = $1)"#,
        filter_active.map(|v| v as i16),
    )
    .fetch_all(pool)
    .await?)
}

pub async fn create_department(pool: &PgPool, department: Department) -> Result<(), ApiError> {
    let mut transaction = pool.begin().await?;

    sqlx::query!(
        r#"INSERT INTO departments
        ("DepartmentName", "Description", "Status", "ManagerID", "ViceManagerID") 
        VALUES ($1, $2, $3, $4, $5)"#,
        department.name,
        department.description,
        department.status as i16,
        department.manager.0.unwrap_or_default().user_id,
        department.vice_manager.0.unwrap_or_default().user_id
    )
    .execute(&mut *transaction)
    .await?;

    // commit transaction
    transaction.commit().await?;

    Ok(())
}

pub async fn update_department(pool: &PgPool, department: Department) -> Result<(), ApiError> {
    let mut transaction = pool.begin().await?;

    sqlx::query!(
        r#"UPDATE departments SET
            "DepartmentName" = $2,
            "Description" = $3,
            "Status" = $4,
            "ManagerID" = $5,
            "ViceManagerID" = $6
        WHERE "DepartmentID" = $1"#,
        department.id,
        department.name,
        department.description,
        department.status as i16,
        department.manager.0.unwrap_or_default().user_id,
        department.vice_manager.0.unwrap_or_default().user_id,
    )
    .execute(&mut *transaction)
    .await?;

    // commit transaction
    transaction.commit().await?;

    Ok(())
}
