use axum::{debug_handler, Json};
use chrono::NaiveDateTime;
use serde::Deserialize;
use ts_rs::TS;
use utoipa::ToSchema;

use crate::{
    api::{
        description::FILE_TAG,
        routes::{files::sql, AuthBackendType},
    },
    errors::ApiError,
};

#[derive(Debug, Deserialize, TS, ToSchema)]
#[ts(export)]
pub struct GetDeleteFileRequest {
    draft_id: String,
    hash: String,
}

#[debug_handler]
#[utoipa::path(
    delete,
    path = "/files/draft",
    summary = "Delete specified file attached to Draft",
    description = "Delete the specified draft file.",
    request_body(content = GetDeleteFileRequest, description = "Request for file deletion", content_type = "application/json"),
    responses(
        (status = OK, description = "File successfully deleted"),
    ),
     tag = FILE_TAG)]
pub async fn delete_draft_file(
    auth_session: AuthBackendType,
    Json(request): Json<GetDeleteFileRequest>,
) -> Result<(), ApiError> {
    let backend = auth_session.backend();
    let user = auth_session
        .is_authenticated()
        .ok_or(ApiError::AccessDenied)?;

    let draft_id =
        NaiveDateTime::parse_from_str(&request.draft_id, "%Y-%m-%d %H:%M:%S").map_err(|_| {
            ApiError::InvalidRequest("Could not parse NaiveDateTime from draft id".to_string())
        })?;

    sql::delete_draft_file(backend.pool(), user, draft_id, request.hash).await
}
