use axum::{debug_handler, Json};
use chrono::NaiveDateTime;
use serde::Deserialize;
use ts_rs::TS;
use utoipa::ToSchema;

use crate::{
    api::{
        description::FILE_TAG,
        routes::{
            files::{models::AttachedFile, sql},
            AuthBackendType,
        },
    },
    errors::ApiError,
};

#[derive(Debug, Deserialize, TS, ToSchema)]
#[ts(export)]
pub struct GetAttachedDraftFilesRequest {
    draft_id: String,
}

#[debug_handler]
#[utoipa::path(
    get,
    path = "/files/draft",
    summary = "Get Files attached to Draft",
    description = "Get list of all files that are attached to a specified draft.",
    request_body(content = GetAttachedDraftFilesRequest, description = "Request for attached files", content_type = "application/json"),
    responses(
        (status = OK, body = AttachedFile, description = "Attached Files List", content_type = "application/json"),
    ),
     tag = FILE_TAG)]
pub async fn get_attached_draft_files(
    auth_session: AuthBackendType,
    Json(request): Json<GetAttachedDraftFilesRequest>,
) -> Result<Json<Vec<AttachedFile>>, ApiError> {
    let backend = auth_session.backend();
    let user = auth_session
        .is_authenticated()
        .ok_or(ApiError::AccessDenied)?;

    let draft_id =
        NaiveDateTime::parse_from_str(&request.draft_id, "%Y-%m-%d %H:%M:%S").map_err(|_| {
            ApiError::InvalidRequest("Could not parse NaiveDateTime from draft id".to_string())
        })?;
    Ok(Json(
        sql::get_draft_attached_files(backend.pool(), user, draft_id).await?,
    ))
}
