use axum::{
    debug_handler,
    extract::{Multipart, Query},
};
use chrono::NaiveDateTime;
use serde::Deserialize;
use ts_rs::TS;
use utoipa::{schema, IntoParams, ToSchema};

use crate::{
    api::{
        description::FILE_TAG,
        routes::{
            files::{handlers::parse_file_from_multipart_form, sql},
            AuthBackendType,
        },
    },
    errors::ApiError,
};

use super::super::FILE_SIZE_LIMIT_MB;

#[derive(Debug, Clone, Deserialize, TS, ToSchema)]
#[ts(export)]
#[allow(unused)]
pub struct FileUploadRequest {
    name: Option<String>,
    #[schema(format = Binary, content_media_type = "application/octet-stream")]
    file: String,
}

#[derive(Debug, Deserialize, IntoParams, ToSchema)]
pub struct DraftFileUploadQueryParams {
    draft_id: String,
}

#[debug_handler]
#[utoipa::path(
    post,
    path = "/files/draft",
    summary = "Draft File Upload",
    description = "Upload a file as draft file.",
    params(DraftFileUploadQueryParams),
    request_body(content = FileUploadRequest, description = "File Data", content_type = "multipart/form-data"),
    responses(
        (status = OK, body = String, description = "Successfully uploaded and stored file"),
        (status = 413, description = format!("The size of the uploaded file is too large (max {FILE_SIZE_LIMIT_MB} MB)"))
    ),
     tag = FILE_TAG)]
pub async fn upload_draft_file(
    auth_session: AuthBackendType,
    Query(parameters): Query<DraftFileUploadQueryParams>,
    mut multipart: Multipart,
) -> Result<(), ApiError> {
    let backend = auth_session.backend();
    let user = auth_session
        .is_authenticated()
        .ok_or(ApiError::AccessDenied)?;

    // get draft id
    let draft_id = NaiveDateTime::parse_from_str(&parameters.draft_id, "%Y-%m-%d %H:%M:%S")
        .map_err(|_| {
            ApiError::InvalidRequest("Could not parse NaiveDateTime from draft id".to_string())
        })?;

    let file = parse_file_from_multipart_form(&mut multipart).await?;
    sql::insert_new_draft_file(backend.pool(), user, draft_id, &file).await?;

    Ok(())
}
