use axum::{
    body::Body,
    debug_handler,
    http::header,
    response::{IntoResponse, Response},
    Json,
};
use serde::Deserialize;
use ts_rs::TS;
use utoipa::ToSchema;

use crate::{
    api::{
        description::FILE_TAG,
        routes::{files::sql, AuthBackendType},
    },
    errors::ApiError,
};

#[derive(Debug, Deserialize, TS, ToSchema)]
#[ts(export)]
pub struct GetFileRequest {
    name: String,
    hash: String,
}

#[debug_handler]
#[utoipa::path(
    get,
    path = "/files/file",
    summary = "File Download",
    description = "Download specified file.",
    request_body(content = GetFileRequest, description = "Request to download specified file", content_type = "application/json"),
    responses(
        (status = OK, description = "File Data", content_type = "application/octet-stream"),
    ),
     tag = FILE_TAG)]
pub async fn get_specified_file(
    auth_session: AuthBackendType,
    Json(request): Json<GetFileRequest>,
) -> Result<impl IntoResponse, ApiError> {
    let backend = auth_session.backend();

    match sql::get_specified_file(backend.pool(), request.name, request.hash).await? {
        Some(file) => Ok(Response::builder()
            .header(header::CONTENT_TYPE, file.content_type)
            .header(
                header::CONTENT_DISPOSITION,
                format!("attachment; filename=\"{}\"", file.name),
            )
            .body(Body::from(file.data))
            .unwrap_or_default()),
        None => Err(ApiError::FileNotFound),
    }
}
