use axum::{
    body::Body,
    debug_handler,
    http::{header, StatusCode},
    response::{IntoResponse, Response},
    Json,
};

use crate::{
    api::{
        description::FILE_TAG,
        routes::{
            files::{
                handlers::{create_zip_bundle, files_order_get::GetAttachedOrderFilesRequest},
                sql,
            },
            AuthBackendType,
        },
    },
    errors::ApiError,
};

#[debug_handler]
#[utoipa::path(
    get,
    path = "/files/order/bundle",
    summary = "Get Files attached to Order Position as ZIP bundle",
    description = "Downloads a ZIP bundle of alle the files attached to the order.",
    request_body(content = GetAttachedOrderFilesRequest, description = "Request for attached files ZIP bundle", content_type = "application/json"),
    responses(
        (status = OK, description = "ZIP bundle Data", content_type = "application/octet-stream"),
    ),
     tag = FILE_TAG)]
pub async fn get_attached_order_files_zip_bundle(
    auth_session: AuthBackendType,
    Json(request): Json<GetAttachedOrderFilesRequest>,
) -> Result<impl IntoResponse, ApiError> {
    let backend = auth_session.backend();

    // get all attached files with data
    let files =
        sql::get_order_attached_files_data(backend.pool(), request.order_id, request.position_id)
            .await?;

    if files.len() == 0 {
        return Ok(Response::builder()
            .status(StatusCode::NO_CONTENT)
            .body(Body::from(""))
            .unwrap_or_default());
    }

    // build zip file
    let zip_data = create_zip_bundle(files)?;

    Ok(Response::builder()
        .header(header::CONTENT_TYPE, "application/x-zip-compressed")
        .header(
            header::CONTENT_DISPOSITION,
            format!(
                "attachment; filename=\"{}\"",
                format!("{}-{}.zip", request.order_id, request.position_id)
            ),
        )
        .body(Body::from(zip_data))
        .unwrap_or_default())
}
