use axum::{debug_handler, Json};
use serde::Deserialize;
use ts_rs::TS;
use utoipa::ToSchema;

use crate::{
    api::{
        description::FILE_TAG,
        routes::{
            files::{models::AttachedFile, sql},
            AuthBackendType,
        },
    },
    errors::ApiError,
};

#[derive(Debug, Deserialize, TS, ToSchema)]
#[ts(export)]
pub struct GetAttachedOrderFilesRequest {
    pub order_id: i32,
    pub position_id: i32,
}

#[debug_handler]
#[utoipa::path(
    get,
    path = "/files/order",
    summary = "Get Files attached to Order Position",
    description = "Get list of all files that are attached to a specified Position.",
    request_body(content = GetAttachedOrderFilesRequest, description = "Request for attached files", content_type = "application/json"),
    responses(
        (status = OK, body = AttachedFile, description = "Attached Files List", content_type = "application/json"),
    ),
     tag = FILE_TAG)]
pub async fn get_attached_order_files(
    auth_session: AuthBackendType,
    Json(request): Json<GetAttachedOrderFilesRequest>,
) -> Result<Json<Vec<AttachedFile>>, ApiError> {
    let backend = auth_session.backend();

    Ok(Json(
        sql::get_order_attached_files(backend.pool(), request.order_id, request.position_id)
            .await?,
    ))
}
