use axum::{
    debug_handler,
    extract::{Multipart, Query},
};
use serde::Deserialize;
use utoipa::{IntoParams, ToSchema};

use crate::{
    api::{
        description::FILE_TAG,
        routes::{
            files::{handlers::files_draft_post::FileUploadRequest, sql},
            AuthBackendType,
        },
    },
    errors::ApiError,
};

use super::{super::FILE_SIZE_LIMIT_MB, parse_file_from_multipart_form};

#[derive(Debug, Deserialize, IntoParams, ToSchema)]
pub struct OrderFileUploadQueryParams {
    pub order_id: i32,
    pub position_id: i32,
}

#[debug_handler]
#[utoipa::path(
    post,
    path = "/files/order",
    summary = "Order File Upload",
    description = "Upload a file to an Order.",
    params(OrderFileUploadQueryParams),
    request_body(content = FileUploadRequest, description = "File Data", content_type = "multipart/form-data"),
    responses(
        (status = OK, body = String, description = "Successfully uploaded and stored file"),
        (status = 413, description = format!("The size of the uploaded file is too large (max {FILE_SIZE_LIMIT_MB} MB)"))
    ),
     tag = FILE_TAG)]
pub async fn upload_order_file(
    auth_session: AuthBackendType,
    Query(parameters): Query<OrderFileUploadQueryParams>,
    mut multipart: Multipart,
) -> Result<String, ApiError> {
    let backend = auth_session.backend();

    let file = parse_file_from_multipart_form(&mut multipart).await?;
    sql::insert_new_order_file(backend.pool(), parameters, &file).await?;

    Ok(file.hash())
}
