use std::io::Write;

use axum::{extract::Multipart, http::StatusCode};
use tokio_util::bytes::Bytes;
use zip::{write::SimpleFileOptions, ZipWriter};

use crate::errors::ApiError;

use super::models::File;

pub mod files_draft_delete;
pub mod files_draft_get;
pub mod files_draft_post;
pub mod files_file_get;
pub mod files_order_bundle_get;
pub mod files_order_get;
pub mod files_order_post;

pub async fn parse_file_from_multipart_form(multipart: &mut Multipart) -> Result<File, ApiError> {
    let mut name: Option<String> = None;

    let mut content_type: Option<String> = None;
    let mut file_name: Option<String> = None;
    let mut bytes: Option<Bytes> = None;
    let mut size: Option<i32> = None;

    while let Some(field) = multipart.next_field().await.unwrap() {
        let field_name = field.name();

        match &field_name {
            Some("name") => name = Some(field.text().await?),
            Some("file") => {
                file_name = field.file_name().map(ToString::to_string);
                content_type = field.content_type().map(ToString::to_string);
                let _bytes = field.bytes().await?;
                size = Some(_bytes.len() as i32);
                bytes = Some(_bytes);
            }
            _ => (),
        };
    }

    if let (Some(data), Some(content_type), Some(filename)) = (bytes, content_type, file_name) {
        Ok(File {
            name: match name {
                Some(name) => name,
                None => filename,
            },
            content_type,
            data,
            size: size.unwrap_or_default(),
        })
    } else {
        Err(ApiError::MultipartForm(
            StatusCode::BAD_REQUEST,
            "Missing fields in request".to_string(),
        ))
    }
}

pub fn create_zip_bundle(files: Vec<File>) -> Result<Vec<u8>, ApiError> {
    let mut data = std::io::Cursor::new(Vec::new());
    let mut zip = ZipWriter::new(&mut data);
    let options = SimpleFileOptions::default().compression_method(zip::CompressionMethod::Deflated);

    for file in files {
        zip.start_file(file.name, options)?;
        zip.write_all(&file.data)?;
    }

    zip.finish()?;

    Ok(data.into_inner())
}
