use chrono::NaiveDateTime;
use sqlx::{PgPool, Postgres, Transaction};

use crate::{api::routes::users::models::User, errors::ApiError};

use super::{
    handlers::files_order_post::OrderFileUploadQueryParams,
    models::{AttachedFile, File},
};

pub async fn insert_new_draft_file(
    pool: &PgPool,
    user: &User,
    draft_id: NaiveDateTime,
    file: &File,
) -> Result<(), ApiError> {
    let mut transaction = pool.begin().await?;

    // add file
    let affected_rows = insert_new_file(&mut transaction, &file).await?;

    // add file <-> draft connection
    let added_connections = sqlx::query!(
        r#"INSERT INTO drafts_files
        ("FileHash", "FileName", "DraftIDTime", "DraftIDUser")
        VALUES ($1, $2, $3, $4) ON CONFLICT DO NOTHING"#,
        file.hash(),
        file.name,
        draft_id,
        user.user_id
    )
    .execute(&mut *transaction)
    .await?
    .rows_affected();

    // commit transaction
    transaction.commit().await?;

    Ok(())
}

pub async fn get_draft_attached_files(
    pool: &PgPool,
    user: &User,
    draft_id: NaiveDateTime,
) -> Result<Vec<AttachedFile>, ApiError> {
    Ok(sqlx::query_as!(
        AttachedFile,
        r#"SELECT 
            "FileHash" as hash,
            "ContentType" as content_type,
            "FileName" as name,
            "Size" as size
        FROM 
            "drafts_files"
        LEFT JOIN files ON
            FILES."Hash" = DRAFTS_FILES."FileHash"
        WHERE
            DRAFTS_FILES."DraftIDUser" = $1 AND "DraftIDTime" = $2
        ORDER BY "FileName" ASC"#,
        user.user_id,
        draft_id
    )
    .fetch_all(pool)
    .await?)
}

pub async fn delete_draft_file(
    pool: &PgPool,
    user: &User,
    draft_id: NaiveDateTime,
    hash: String,
) -> Result<(), ApiError> {
    let mut transaction = pool.begin().await?;

    // delete draft file
    sqlx::query!(
        r#"DELETE FROM 
            "drafts_files" 
        WHERE 
            "DraftIDUser" = $1 
            AND "DraftIDTime" = $2 
            AND "FileHash" = $3"#,
        user.user_id,
        draft_id,
        hash
    )
    .execute(&mut *transaction)
    .await?;

    // check if file is now dangling and can be deleted
    if check_dangling_file(&mut transaction, &hash).await? {
        delete_file(&mut transaction, &hash).await?;
    }

    // commit transaction
    transaction.commit().await?;

    Ok(())
}

pub async fn get_specified_file(
    pool: &PgPool,
    name: String,
    hash: String,
) -> Result<Option<File>, ApiError> {
    let file = sqlx::query_as!(
        File,
        r#"SELECT 
            "ContentType" as content_type,
            $2 as "name!: String",
            "Data" as data,
            "Size" as size
        FROM
            files
        WHERE
            "Hash" = $1"#,
        hash,
        name
    )
    .fetch_optional(pool)
    .await?;

    Ok(file)
}

pub async fn insert_new_order_file(
    pool: &PgPool,
    order: OrderFileUploadQueryParams,
    file: &File,
) -> Result<(), ApiError> {
    let mut transaction = pool.begin().await?;
    // add file
    insert_new_file(&mut transaction, &file).await?;

    // add file <-> draft connection
    let added_connections = sqlx::query!(
        r#"INSERT INTO "orderPositions_files"
        ("FileHash", "FileName", "OrderID", "PositionID")
        VALUES ($1, $2, $3, $4) ON CONFLICT DO NOTHING"#,
        file.hash(),
        file.name,
        order.order_id,
        order.position_id,
    )
    .execute(&mut *transaction)
    .await?
    .rows_affected();

    // commit transaction
    transaction.commit().await?;

    Ok(())
}

pub async fn get_order_attached_files(
    pool: &PgPool,
    order_id: i32,
    position_id: i32,
) -> Result<Vec<AttachedFile>, ApiError> {
    Ok(sqlx::query_as!(
        AttachedFile,
        r#"SELECT 
            "FileHash" as hash,
            "ContentType" as content_type,
            "FileName" as name,
            "Size" as size
        FROM 
            "orderPositions_files"
        LEFT JOIN files ON
            FILES."Hash" = "orderPositions_files"."FileHash"
        WHERE
             "orderPositions_files"."OrderID" = $1 AND  "orderPositions_files"."PositionID" = $2
        ORDER BY "FileName" ASC"#,
        order_id,
        position_id
    )
    .fetch_all(pool)
    .await?)
}

pub async fn get_order_attached_files_data(
    pool: &PgPool,
    order_id: i32,
    position_id: i32,
) -> Result<Vec<File>, ApiError> {
    Ok(sqlx::query_as!(
        File,
        r#"SELECT 
            "ContentType" as content_type,
            "FileName" as name,
            "Size" as size,
            "Data" as data
        FROM 
            "orderPositions_files"
        LEFT JOIN files ON
            FILES."Hash" = "orderPositions_files"."FileHash"
        WHERE
             "orderPositions_files"."OrderID" = $1 AND  "orderPositions_files"."PositionID" = $2
        ORDER BY "FileName" ASC"#,
        order_id,
        position_id
    )
    .fetch_all(pool)
    .await?)
}

pub async fn insert_new_file(
    transaction: &mut Transaction<'static, Postgres>,
    file: &File,
) -> Result<u64, ApiError> {
    Ok(sqlx::query!(
        r#"INSERT INTO files
        ("Hash", "ContentType", "Data", "Size")
        VALUES ($1, $2, $3, $4) ON CONFLICT DO NOTHING"#,
        file.hash(),
        file.content_type,
        &file.data.to_vec(),
        file.size
    )
    .execute(&mut **transaction)
    .await?
    .rows_affected())
}

pub async fn check_dangling_file(
    transaction: &mut Transaction<'static, Postgres>,
    hash: &str,
) -> Result<bool, ApiError> {
    Ok(sqlx::query_scalar!(
        r#"SELECT COUNT(*) as "count!" FROM
        (SELECT "FileHash" FROM drafts_files WHERE "FileHash" = $1
        UNION
        SELECT "FileHash" FROM "orderPositions_files" WHERE "FileHash" = $1)"#,
        hash
    )
    .fetch_one(&mut **transaction)
    .await?
        == 0)
}

pub async fn delete_file(
    transaction: &mut Transaction<'static, Postgres>,
    hash: &str,
) -> Result<(), ApiError> {
    sqlx::query!(r#"DELETE FROM files WHERE "Hash" = $1"#, hash)
        .execute(&mut **transaction)
        .await?;

    Ok(())
}
