use super::super::{models::SigningKeyStatus, sql};
use axum::{debug_handler, Json};

use crate::{
    api::{description::SIGNATURE_TAG, routes::AuthBackendType},
    errors::ApiError,
};

#[debug_handler]
#[utoipa::path(
    get,
    path = "/signature/key",
    summary = "Check for signing key",
    description = "Check if the logged in user has configured a signing key.",
    responses(
        (status = OK, body = SigningKeyStatus, description = "Signature Status"),
    ),
    security(
        ("user_auth" = ["write:signature",]),
    ),
    tag = SIGNATURE_TAG)]
pub async fn check_signing_key(
    auth_session: AuthBackendType,
) -> Result<Json<SigningKeyStatus>, ApiError> {
    let backend = auth_session.backend();
    let user = auth_session
        .is_authenticated()
        .ok_or(ApiError::AccessDenied)?;

    let private_key = sql::get_private_key(backend.pool(), &user.user_id).await?;

    Ok(Json(SigningKeyStatus {
        configured: private_key.key.is_some(),
        verified: None,
    }))
}
