use super::super::sql;
use axum::{debug_handler, Json};
use chrono::Utc;
use minisign::KeyPair;
use serde::Deserialize;
use ts_rs::TS;
use utoipa::ToSchema;

use crate::{
    api::{description::SIGNATURE_TAG, routes::AuthBackendType},
    errors::ApiError,
};

#[derive(Debug, Clone, Deserialize, TS, ToSchema)]
#[ts(export)]
pub struct CreateSigningKeyRequest {
    secret: String,
}

#[debug_handler]
#[utoipa::path(
    post,
    path = "/signature/key",
    summary = "Create signing key",
    description = "Create signing key for the currently logged in user.",
    request_body(content = CreateSigningKeyRequest, description = "Signing Key Creation Request", content_type = "application/json"),
    responses(
        (status = OK, description = "Successfully created signing key"),
    ),
    security(
        ("user_auth" = ["write:signature",]),
    ),
    tag = SIGNATURE_TAG)]
pub async fn create_signing_key(
    auth_session: AuthBackendType,
    Json(key_request): Json<CreateSigningKeyRequest>,
) -> Result<(), ApiError> {
    let backend = auth_session.backend();
    let user = auth_session
        .is_authenticated()
        .ok_or(ApiError::AccessDenied)?;

    // create Key Pair
    let KeyPair { pk, sk } = KeyPair::generate_encrypted_keypair(Some(key_request.secret))?;
    let public_key = pk.to_box()?.to_string();
    let private_key = sk
        .to_box(Some(&format!(
            "{} {} - {}",
            user.name,
            user.surname,
            Utc::now()
        )))? // Optional comment about the key
        .to_string();

    // store keys in database
    sql::update_private_key(backend.pool(), &user.user_id, private_key).await?;
    sql::store_public_key(backend.pool(), &user.user_id, public_key).await?;

    Ok(())
}
