use super::super::models::{SigningKeyRequest, SigningKeyStatus};
use axum::{debug_handler, Json};

use crate::{
    api::{description::SIGNATURE_TAG, routes::AuthBackendType},
    errors::ApiError,
};

#[debug_handler]
#[utoipa::path(
    put,
    path = "/signature/key",
    summary = "Verify signing key",
    description = "Verify signing key for the currently logged in user.",
    request_body(content = SigningKeyRequest, description = "Signing Key Request", content_type = "application/json"),
    responses(
        (status = OK, body = SigningKeyStatus, description = "Signature Status"),
    ),
    security(
        ("user_auth" = ["write:signature",]),
    ),
    tag = SIGNATURE_TAG)]
pub async fn verify_signing_key(
    auth_session: AuthBackendType,
    Json(key_request): Json<SigningKeyRequest>,
) -> Result<Json<SigningKeyStatus>, ApiError> {
    let backend = auth_session.backend();
    let user = auth_session
        .is_authenticated()
        .ok_or(ApiError::AccessDenied)?;

    // get private key
    let status = match key_request
        .to_private_key(backend.pool(), &user.user_id, backend.private_key_cache())
        .await?
    {
        Some(_key) => SigningKeyStatus {
            configured: true,
            verified: Some(true),
        },
        None => SigningKeyStatus {
            configured: false,
            verified: None,
        },
    };

    Ok(Json(status))
}
