use std::io::Cursor;

use super::super::{
    models::{SignatureTest, SigningKeyRequest, VerificationRequest},
    sql,
};
use axum::{debug_handler, Json};

use crate::{
    api::{description::SIGNATURE_TAG, routes::AuthBackendType},
    errors::ApiError,
};

#[debug_handler]
#[utoipa::path(
    post,
    path = "/signature/sign",
    summary = "Sign something with your signing key",
    description = "Sign something as the currently logged in user.",
    request_body(content = SigningKeyRequest, description = "Signing Key Request", content_type = "application/json"),
    responses(
        (status = OK, description = "Successfully signed"),
        (status = 401, description = "Unauthorized"),
        (status = 500, description = "Internal Server Error")
    ),
    security(
        ("user_auth" = ["write:signature",]),
    ),
    tag = SIGNATURE_TAG)]
pub async fn sign(
    auth_session: AuthBackendType,
    Json(key_request): Json<SigningKeyRequest>,
) -> Result<Json<VerificationRequest>, ApiError> {
    let backend = auth_session.backend();
    let user = auth_session
        .is_authenticated()
        .ok_or(ApiError::AccessDenied)?;

    // get private key
    let private_key = key_request
        .to_private_key(backend.pool(), &user.user_id, backend.private_key_cache())
        .await?
        .ok_or(ApiError::AccessDenied)?;

    // get current public key
    let public_key_id = sql::get_current_public_key(backend.pool(), &user.user_id)
        .await?
        .ok_or(ApiError::InternalError(
            "No corresponding public key found".to_string(),
        ))?;

    let data = SignatureTest {
        value1: "blfjdljfa".to_string(),
        value2: false,
        value3: vec!["bladfa".to_string(), "noch bla".to_string()],
    };

    let data_reader = Cursor::new(data.calculate_hash().to_be_bytes());
    let signature_box = minisign::sign(
        None,
        &private_key,
        data_reader,
        None,
        Some(&serde_json::to_string(&public_key_id)?),
    )?;

    // We have a signature! Let's inspect it a little bit.
    println!(
        "Untrusted comment: [{}]",
        signature_box.untrusted_comment().unwrap()
    );
    println!(
        "Trusted comment: [{}]",
        signature_box.trusted_comment().unwrap()
    );

    // store signature
    // Converting the signature box to a string in order to save it is easy.
    let signature = signature_box.into_string();
    println!("BLA");
    println!("{signature}");

    Ok(Json(VerificationRequest {
        signature,
        signed_from: "".to_string(),
    }))
}
