use std::io::Cursor;

use super::super::{
    models::{SignatureTest, SigningKeyRequest, VerificationRequest},
    sql,
};
use axum::{debug_handler, Json};
use minisign::{PublicKeyBox, SignatureBox};

use crate::{
    api::{
        description::SIGNATURE_TAG,
        routes::{signature::models::PublicKeyID, AuthBackendType},
    },
    errors::ApiError,
};

#[debug_handler]
#[utoipa::path(
    get,
    path = "/signature/verify",
    summary = "Verify a signature",
    description = "Verify the signature.",
    request_body(content = SigningKeyRequest, description = "Signing Key Request", content_type = "application/json"),
    responses(
        (status = OK, description = "Successfully signed"),
    ),
    tag = SIGNATURE_TAG)]
pub async fn verify(
    auth_session: AuthBackendType,
    Json(verification_request): Json<VerificationRequest>,
) -> Result<(), ApiError> {
    let backend = auth_session.backend();

    // Now, let's verify the signature.
    // Assuming we just loaded it into `signature_box_str`, get the box back.
    let signature_box = SignatureBox::from_string(&verification_request.signature)?;

    // get public key id from signature
    let public_key_id: PublicKeyID = serde_json::from_str(&signature_box.untrusted_comment()?)?;

    // Load the public key of the user that signed.
    let public_key_str = sql::get_public_key_by_id(backend.pool(), public_key_id)
        .await?
        .key
        .ok_or(ApiError::InternalError(
            "No corresponding Public Key was found.".to_string(),
        ))?;
    let public_key_box = PublicKeyBox::from_string(&public_key_str)?;
    let public_key = public_key_box.into_public_key()?;

    // And verify the data.
    let data = SignatureTest {
        value1: "blfjdljfa".to_string(),
        value2: false,
        value3: vec!["bladfa".to_string(), "noch bla".to_string()],
    };
    let data_reader = Cursor::new(data.calculate_hash().to_be_bytes());
    let verified = minisign::verify(&public_key, &signature_box, data_reader, true, false, false);
    match verified {
        Ok(()) => println!("Success!"),
        Err(_) => println!("Verification failed"),
    };

    Ok(())
}
