use std::{
    hash::{DefaultHasher, Hash, Hasher},
    time::Duration,
};

use chrono::NaiveDateTime;
use minisign::{SecretKey, SecretKeyBox};
use serde::{Deserialize, Serialize};
use sqlx::PgPool;
use ts_rs::TS;
use utoipa::ToSchema;

use crate::{
    api::{backend::private_key_cache::PrivateKeyCache, routes::signature::sql},
    errors::ApiError,
};

#[derive(Debug, Clone, Serialize, Deserialize, TS, ToSchema)]
#[ts(export)]
pub struct SigningKeyStatus {
    pub configured: bool,
    pub verified: Option<bool>,
}

pub struct KeyData {
    pub key: Option<String>,
}

#[derive(Debug, Clone, Serialize, Deserialize)]
pub struct PublicKeyID {
    pub timestamp: NaiveDateTime,
    pub user_id: String,
}

#[derive(Debug, Clone, Deserialize, TS, ToSchema)]
#[ts(export)]
pub struct SigningKeyRequest {
    // if no secret is given, the key will be searched in the cache
    pub secret: Option<String>,
    // if ttl is set, the key is cached for ttl in seconds
    pub ttl: Option<u32>,
}

impl SigningKeyRequest {
    pub async fn to_private_key(
        self,
        pool: &PgPool,
        user_id: &str,
        private_key_cache: PrivateKeyCache,
    ) -> Result<Option<SecretKey>, ApiError> {
        // get private key from database and decrypt it or get it from cache
        // this depends on the request containing a secret
        // if a secret and a TTL is given the decrypted private key is cached
        let private_key = match self.secret {
            Some(secret) => {
                // try to get private key from database
                let encrypted_private_key = match sql::get_private_key(pool, user_id).await?.key {
                    Some(key) => key,
                    None => return Ok(None),
                };

                // create MinSign Box from key string
                let private_key_box = SecretKeyBox::from_string(&encrypted_private_key)?;
                // and the box can be opened using the password to reveal the original secret key:
                let private_key = private_key_box.into_secret_key(Some(secret))?;

                // cache key if TTL is set
                if let Some(ttl) = self.ttl {
                    let duration = Duration::from_secs(ttl.into());
                    private_key_cache
                        .add_key(user_id, &private_key, duration)
                        .await;
                }

                private_key
            }
            None => match private_key_cache.get_key(user_id).await {
                Some(key) => key.secret_key(),
                None => return Err(ApiError::InvalidCredentials),
            },
        };

        Ok(Some(private_key))
    }
}

#[derive(Debug, Clone, Deserialize, Serialize, TS, ToSchema)]
#[ts(export)]
pub struct VerificationRequest {
    pub signature: String,
    pub signed_from: String,
}

#[derive(Debug, Hash)]
pub struct SignatureTest {
    pub value1: String,
    pub value2: bool,
    pub value3: Vec<String>,
}

impl SignatureTest
where
    Self: Hash,
{
    pub fn calculate_hash(&self) -> u64 {
        let mut s = DefaultHasher::new();
        self.hash(&mut s);
        s.finish()
    }
}
