use axum::{debug_handler, Extension, Json};
use axum_jwt_login::UserPermissions;

use crate::{
    api::{
        backend::{
            ldap::{ActiveDirectoryUser, LDAPBackend},
            ApiBackend,
        },
        description::USERS_TAG,
        routes::{auth::models::Credentials, users::sql},
    },
    errors::ApiError,
};

#[debug_handler]
#[utoipa::path(
    get,
    path = "/users/available_ad_users",
    summary = "Get Active Directory Users",
    description = "Get all Available Users from the Active Directory that are not already registered with this API.",
    responses(
        (status = OK, body = Vec<ActiveDirectoryUser>, description = "List of AD users"),
    ),
    security(
        ("user_auth" = ["write:users",]),
    ),
    tag = USERS_TAG)]
pub async fn get_ad_users(
    Extension(backend): Extension<ApiBackend>,
    Json(credentials): Json<Option<Credentials>>,
) -> Result<Json<Vec<ActiveDirectoryUser>>, ApiError> {
    let api_user_ids: Vec<String> = sql::get_users(backend.pool(), None, None)
        .await?
        .iter()
        .map(|user| user.id())
        .collect();
    let mut ldap = LDAPBackend::from_config(backend.config()).await?;
    // bind to AD user if credentials are given
    if let Some(credentials) = credentials {
        ldap.ad_bind(&credentials.id, &credentials.password).await?;
    }
    let ad_users = ldap
        .get_ad_user_list()
        .await?
        .into_iter()
        .filter(|entry| !api_user_ids.contains(&entry.id))
        .collect();
    // disconnect from AD server
    ldap.unbind().await;

    Ok(Json(ad_users))
}
