use axum::{debug_handler, Extension, Json};

use super::super::sql;
use crate::{
    api::{backend::ApiBackend, description::USERS_TAG, routes::users::models::User},
    errors::ApiError,
    utils::create_random,
};

#[debug_handler]
#[utoipa::path(
    post,
    path = "/users",
    summary = "Create a new User",
    description = "Creates a new user with the given information ",
    request_body(content = User, description = "User details", content_type = "application/json"),
    responses(
        (status = OK, description = "User successfully created (Assigned Password in body)", body = String),
    ),
    security(
        ("user_auth" = ["write:users",]),
    ),
    tag = USERS_TAG)]
pub async fn create_user(
    Extension(backend): Extension<ApiBackend>,
    Json(user): Json<User>,
) -> Result<String, ApiError> {
    // create password if not Active Directory user
    let (password, hash) = match user.active_directory_auth {
        true => (String::new(), None),
        false => {
            let salt = create_random(20);
            let argon_config = argon2::Config::default();

            let password = create_random(10);
            let password_hash =
                argon2::hash_encoded(password.as_bytes(), salt.as_bytes(), &argon_config)?;

            (password, Some(password_hash))
        }
    };
    // create user
    sql::create_new_user(backend.pool(), &user, hash).await?;
    // send created password back to frontend
    Ok(password)
}
