use std::{
    collections::{HashMap, HashSet},
    fmt,
};

use axum_jwt_login::UserPermissions;
use chrono::NaiveDateTime;
use serde::{Deserialize, Serialize};
use sqlx::{types::JsonValue, PgPool};
use tracing::error;
use ts_rs::TS;
use utoipa::ToSchema;

use crate::{
    api::routes::{departments::models::ShortDepartment, models::Status},
    errors::ApiError,
};

use super::{
    permissions::{Permission, PermissionContainer},
    sql,
};

#[derive(Debug, Clone, Serialize, Deserialize, TS, ToSchema)]
pub struct GroupContainer(pub HashSet<i32>);

impl From<Option<Vec<i32>>> for GroupContainer {
    fn from(value: Option<Vec<i32>>) -> Self {
        let set = match value {
            Some(values) => HashSet::from_iter(values),
            None => HashSet::new(),
        };

        Self(set)
    }
}

#[derive(Debug, Clone, Serialize, Deserialize, TS, ToSchema)]
pub struct DepartmentContainer(pub Option<ShortDepartment>);

impl From<Option<JsonValue>> for DepartmentContainer {
    fn from(value: Option<JsonValue>) -> Self {
        Self(
            value
                .map(|value| -> Option<ShortDepartment> {
                    match serde_json::from_value::<ShortDepartment>(value) {
                        Ok(dep) => Some(dep),
                        Err(err) => {
                            error!("{err:?}");
                            None
                        }
                    }
                })
                .flatten(),
        )
    }
}

#[derive(Clone, Serialize, Deserialize, TS, ToSchema)]
#[ts(export)]
pub struct User {
    pub user_id: String,
    pub active_directory_auth: bool,
    #[serde(skip)] // do not leak password hashes
    pub password: String,
    pub name: String,
    pub surname: String,
    pub email: String,
    #[schema(inline)]
    pub department: DepartmentContainer,
    #[schema(inline)]
    pub groups: GroupContainer,
    #[schema(inline)]
    pub group_permissions: PermissionContainer,
    #[schema(inline)]
    pub permissions: PermissionContainer,
    pub status_flag: Status,
    #[serde(default)]
    #[schema(value_type = String, read_only)]
    pub creation_date: NaiveDateTime,
    #[serde(default)]
    #[schema(value_type = String, read_only)]
    pub last_change: NaiveDateTime,
}

/// Manually implement Debug to prevent password leaking
impl fmt::Debug for User {
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        f.debug_struct("User")
            .field("user_id", &self.user_id)
            .field("active_directory_auth", &self.active_directory_auth)
            .field("password", &"[redacted]")
            .field("name", &self.name)
            .field("surname", &self.surname)
            .field("email", &self.email)
            .field("department", &self.department)
            .field("groups", &self.groups)
            .field("group_permissions", &self.group_permissions)
            .field("permissions", &self.permissions)
            .field("status_flag", &self.status_flag)
            .field("creation_date", &self.creation_date)
            .field("last_change", &self.last_change)
            .finish()
    }
}

impl UserPermissions for User {
    type Error = ApiError;
    type Permission = Permission;
    type Id = String;

    fn id(&self) -> Self::Id {
        self.user_id.clone()
    }

    fn get_user_permissions(&self) -> Result<HashSet<Self::Permission>, Self::Error> {
        Ok(self.permissions.0.iter().cloned().collect())
    }
    fn get_group_permissions(&self) -> Result<HashSet<Self::Permission>, Self::Error> {
        Ok(self.group_permissions.0.iter().cloned().collect())
    }
}

impl User {
    pub async fn update_with_ad_details(
        &self,
        pool: &PgPool,
        details: HashMap<String, Vec<String>>,
    ) -> Result<(), ApiError> {
        let create_error = |key: &str| {
            ApiError::InternalError(format!(
                "Update User Information: Information '{key}' is not present"
            ))
        };

        // get field values
        let name = details
            .get("givenName")
            .ok_or(create_error("Name"))?
            .first()
            .ok_or(create_error("Name"))?;
        let surname = details
            .get("sn")
            .ok_or(create_error("Surname"))?
            .first()
            .ok_or(create_error("Surname"))?;
        let email = details
            .get("mail")
            .ok_or(create_error("Email"))?
            .first()
            .ok_or(create_error("Email"))?;

        if name != &self.name || surname != &self.surname || email != &self.email {
            let mut transaction = pool.begin().await?;
            sql::update_user_details(&mut transaction, &self.id(), name, surname, email).await?;
            transaction.commit().await?;
        }

        Ok(())
    }
}

#[derive(Debug, Clone, Default, Serialize, Deserialize, TS, ToSchema)]
#[ts(export)]
pub struct ShortUser {
    pub user_id: String,
    pub name: String,
    pub surname: String,
    pub email: String,
}
