use std::sync::Arc;

use axum::{debug_handler, extract::Multipart, Extension};
use tracing::error;

use super::super::sql;
use crate::{
    api::{
        backend::ApiBackend,
        description::VENDORS_TAG,
        routes::{
            data_hub::models::{DataHub, DataHubData},
            files::handlers::parse_file_from_multipart_form,
        },
    },
    errors::ApiError,
};

const SKIP_TOP: usize = 4;
const SKIP_BOTTOM: usize = 3;

const COLUMN_ID: usize = 1;
const COLUMN_NAME: usize = 2;

#[debug_handler]
#[utoipa::path(
    post,
    path = "/vendors/import",
    summary = "Import Vendors from CSV",
    description = "Import Vendors from an uploaded CSV file.",
    responses(
        (status = OK, description = "Successfully imported vendors"),
    ),
    tag = VENDORS_TAG)]
pub async fn import_vendors(
    Extension(backend): Extension<ApiBackend>,
    Extension(data_hub): Extension<Arc<DataHub>>,
    mut multipart: Multipart,
) -> Result<(), ApiError> {
    let file = parse_file_from_multipart_form(&mut multipart).await?;
    let file_content = String::from_utf8(file.data.to_vec()).map_err(|e| {
        error!("{e}");
        ApiError::InvalidRequest("Invalid file content".to_string())
    })?;

    // skip header lines
    let lines = file_content.lines().into_iter().skip(SKIP_TOP.into());
    let line_count = lines.clone().count();

    // begin transaction
    let mut transaction = backend.pool().begin().await?;

    // get data lines (skipping footer lines)
    for line in lines.take(line_count - SKIP_BOTTOM as usize) {
        // get columns
        let columns = line.split('\t').collect::<Vec<&str>>();

        let id = columns[COLUMN_ID];
        let name = columns[COLUMN_NAME];

        sql::import_vendor_line(&mut transaction, id, name).await?;
    }

    // commit to db
    transaction.commit().await?;

    // get updated unit list
    let vendors = sql::get_vendors(backend.pool()).await?;
    let _ = data_hub.sender.send(DataHubData::Vendors(vendors));

    Ok(())
}
