use std::time::Duration;

use chrono::{DateTime, TimeZone, Utc};
use jsonwebtoken::{decode, Algorithm, DecodingKey, EncodingKey, Header, Validation};
use serde::{Deserialize, Serialize};

use super::user::User;

// Session length in seconds -> 12 hours
pub(crate) const DEFAULT_SESSION_LENGTH: std::time::Duration = Duration::from_secs(12 * 3600);

#[derive(Clone)]
pub struct JwtAuthorizer {
    encoding_key: EncodingKey,
    decoding_key: DecodingKey,
    algorithm: Algorithm,
}

// Here we've implemented `Debug` manually to avoid accidentally logging the keys.
impl std::fmt::Debug for JwtAuthorizer {
    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
        f.debug_struct("JwtAuthorizer")
            .field("encoding_key", &"[redacted]")
            .field("decoding_key", &"[redacted]")
            .field("algorithm", &self.algorithm)
            .finish()
    }
}

impl JwtAuthorizer {
    pub fn new(encoding_key: EncodingKey, decoding_key: DecodingKey, algorithm: Algorithm) -> Self {
        Self {
            encoding_key,
            decoding_key,
            algorithm,
        }
    }

    pub fn encode(&self, user: User) -> Result<String, jsonwebtoken::errors::Error> {
        let claims = RegisteredClaims::new(user);
        jsonwebtoken::encode(&Header::default(), &claims, &self.encoding_key)
    }

    pub fn decode(&self, token: &str) -> Result<User, jsonwebtoken::errors::Error> {
        decode::<RegisteredClaims<User>>(
            &token,
            &self.decoding_key,
            &Validation::new(self.algorithm),
        )
        .map(|data| data.claims.data)
    }
}

/// Claims mentioned in the JWT specifications.
///
/// <https://www.rfc-editor.org/rfc/rfc7519#section-4.1>
#[derive(Deserialize, Serialize, Clone, Debug)]
pub struct RegisteredClaims<T> {
    #[serde(skip_serializing_if = "Option::is_none")]
    pub iss: Option<String>,
    #[serde(skip_serializing_if = "Option::is_none")]
    pub sub: Option<String>,
    #[serde(skip_serializing_if = "Option::is_none")]
    pub exp: Option<NumericDate>,
    #[serde(skip_serializing_if = "Option::is_none")]
    pub nbf: Option<NumericDate>,
    #[serde(skip_serializing_if = "Option::is_none")]
    pub iat: Option<NumericDate>,
    #[serde(skip_serializing_if = "Option::is_none")]
    pub jti: Option<String>,
    pub data: T,
}

impl<T> RegisteredClaims<T> {
    pub fn new(data: T) -> Self {
        let now = Utc::now();
        let exp = now + DEFAULT_SESSION_LENGTH;
        Self {
            iss: Some(env!("CARGO_PKG_NAME").to_string()),
            sub: Some("User token".to_string()),
            exp: Some(exp.into()),
            nbf: Some(now.into()),
            iat: Some(now.into()),
            jti: None,
            data,
        }
    }
}

/// The number of seconds from 1970-01-01T00:00:00Z UTC until the specified UTC date/time ignoring leap seconds.
/// (<https://www.rfc-editor.org/rfc/rfc7519#section-2>)
#[derive(Deserialize, Serialize, Clone, PartialEq, Eq, Debug)]
pub struct NumericDate(pub i64);

/// accesses the underlying value
impl From<NumericDate> for i64 {
    fn from(t: NumericDate) -> Self {
        t.0
    }
}

impl From<DateTime<Utc>> for NumericDate {
    fn from(t: DateTime<Utc>) -> Self {
        Self(t.timestamp())
    }
}

impl From<NumericDate> for DateTime<Utc> {
    fn from(t: NumericDate) -> Self {
        Utc.timestamp_opt(t.0, 0).unwrap()
    }
}
