use std::{
    future::Future,
    pin::Pin,
    task::{Context, Poll},
};

use axum::{
    extract::Request,
    http::{self, header::AUTHORIZATION, HeaderMap, HeaderName, Response},
};
use tower_layer::Layer;
use tower_service::Service;
use tracing::Instrument;

use super::{api_key::ApiKeyInfo, AuthenticationBackend, AuthenticationData, UserAuthData};

#[derive(Debug, Clone)]
pub struct AuthenticationLayer<T> {
    backend: AuthenticationBackend<T>,
}

impl<T> AuthenticationLayer<T> {
    pub fn new(backend: AuthenticationBackend<T>) -> Self {
        Self { backend }
    }
}

impl<S, T> Layer<S> for AuthenticationLayer<T>
where
    T: Clone,
{
    type Service = AuthenticationService<S, T>;

    fn layer(&self, service: S) -> Self::Service {
        AuthenticationService {
            backend: self.backend.clone(),
            service,
        }
    }
}

// This service implements the Log behavior
#[derive(Debug, Clone)]
pub struct AuthenticationService<S, T> {
    backend: AuthenticationBackend<T>,
    service: S,
}

impl<ReqBody, ResBody, S, T> Service<Request<ReqBody>> for AuthenticationService<S, T>
where
    S: Service<Request<ReqBody>, Response = Response<ResBody>> + Clone + Send + 'static,
    S::Future: Send + 'static,
    ReqBody: Send + 'static,
    ResBody: Default + Send,
    T: ApiKeyInfo + Send + Sync + Clone + 'static,
{
    type Response = S::Response;
    type Error = S::Error;
    type Future = Pin<Box<dyn Future<Output = Result<Self::Response, Self::Error>> + Send>>;

    #[inline]
    fn poll_ready(&mut self, cx: &mut Context<'_>) -> Poll<Result<(), Self::Error>> {
        self.service.poll_ready(cx)
    }

    fn call(&mut self, mut req: Request<ReqBody>) -> Self::Future {
        let span = tracing::info_span!("call", user.id = tracing::field::Empty);

        let mut backend = self.backend.clone();

        // Because the inner service can panic until ready, we need to ensure we only
        // use the ready service.
        //
        // See: https://docs.rs/tower/latest/tower/trait.Service.html#be-careful-when-cloning-inner-services
        let clone = self.service.clone();
        let mut inner = std::mem::replace(&mut self.service, clone);

        Box::pin(
            async move {
                // check for api key if api key requirement is enabled
                let bearer_required = if let Some(ref header) = backend.api_key_header {
                    match extract_api_key(req.headers(), header) {
                        Some((id, hash)) => {
                            // get api key information
                            if let Some(key) = backend.session_store.get_api_key(&id).await {
                                // validate api key
                                if !key.validate(&hash) {
                                    tracing::error!("API Key invalid");
                                    let mut res = Response::default();
                                    *res.status_mut() = http::StatusCode::UNAUTHORIZED;
                                    return Ok(res);
                                }
                                // check if further authentication is required
                                if key.requires_auth() {
                                    true
                                } else {
                                    // add API Key to authenticated information
                                    backend.authenticated_user =
                                        Some(AuthenticationData::ApiKey(key));
                                    false
                                }
                            } else {
                                let mut res = Response::default();
                                *res.status_mut() = http::StatusCode::UNAUTHORIZED;
                                return Ok(res);
                            }
                        }
                        None => {
                            let mut res = Response::default();
                            *res.status_mut() = http::StatusCode::UNAUTHORIZED;
                            return Ok(res);
                        }
                    }
                } else {
                    true
                };

                // Authenticate with bearer if required
                if bearer_required {
                    // try to extract Authentication Bearer from headers
                    let bearer = extract_bearer(req.headers());
                    // check if bearer is present
                    if let Some(bearer) = bearer {
                        // check if bearer belongs to active session
                        if backend.session_store.includes(&bearer).await {
                            // try to decode bearer
                            match backend.jwt_authorizer.decode(&bearer) {
                                Ok(user) => {
                                    backend.authenticated_user =
                                        Some(AuthenticationData::User(UserAuthData {
                                            user,
                                            token: bearer,
                                        }));

                                    // tracing::Span::current().record("user.id", user.id().to_string());
                                }
                                Err(err) => {
                                    // remove token from session storage
                                    backend.session_store.remove(&bearer).await;
                                    // log error
                                    tracing::error!("Error decoding JWT token: {err}");
                                    // return internal server error
                                    let mut res = Response::default();
                                    *res.status_mut() = http::StatusCode::INTERNAL_SERVER_ERROR;
                                    return Ok(res);
                                }
                            }
                        }
                    }
                }

                req.extensions_mut().insert(backend);

                inner.call(req).await
            }
            .instrument(span),
        )
    }
}

fn extract_bearer(headers: &HeaderMap) -> Option<String> {
    // Check that its a well-formed bearer and return
    headers
        .get(AUTHORIZATION)
        .map(|header| {
            let split = header.to_str().unwrap_or("").split_once(' ');
            let bearer = match split {
                // Found proper bearer
                Some((name, contents)) if name == "Bearer" => Some(contents.to_string()),
                // Found nothing
                _ => None,
            };
            bearer
        })
        .flatten()
}

fn extract_api_key(headers: &HeaderMap, name: &HeaderName) -> Option<(String, String)> {
    // Check that its a well-formed bearer and return
    headers
        .get(name)
        .map(|header| {
            let split = header.to_str().unwrap_or("").split_once(".");
            let api_key = match split {
                Some((id, hash)) => Some((id.to_string(), hash.to_string())),
                _ => None,
            };
            api_key
        })
        .flatten()
}
