pub mod api_key;
mod extract;
mod jwt;
pub mod layer;
mod middleware;
mod sessions;
mod user;

// use api_key::ApiKey;
use axum::http::HeaderName;
use jsonwebtoken::{Algorithm, DecodingKey, EncodingKey};
use jwt::JwtAuthorizer;
use sessions::SessionStore;
use user::User;

#[derive(Debug, Clone)]
pub struct AuthenticationBackend<T> {
    sql_pool: (),
    session_store: SessionStore<T>,
    jwt_authorizer: JwtAuthorizer,
    api_key_header: Option<HeaderName>,
    authenticated_user: Option<AuthenticationData<T>>,
}

impl<T> AuthenticationBackend<T> {
    pub fn is_authenticated(&self) -> bool {
        self.authenticated_user.is_some()
    }
}

#[derive(Debug, Clone)]
pub enum AuthenticationData<T> {
    User(UserAuthData),
    ApiKey(T),
}

#[derive(Debug, Clone)]
pub struct UserAuthData {
    user: User,
    token: String,
}

#[derive(Clone, Default)]
pub struct AuthenticationBackendBuilder<ApiKey> {
    pool: Option<()>,
    api_header_key: Option<HeaderName>,
    loaded_api_keys: Option<Vec<ApiKey>>,
    jwt_decode_key: Option<DecodingKey>,
    jwt_encoding_key: Option<EncodingKey>,
    jwt_algorithm: Option<Algorithm>,
}

impl<ApiKey> AuthenticationBackendBuilder<ApiKey> {
    pub fn new() -> Self {
        Self {
            pool: None,
            api_header_key: None,
            loaded_api_keys: None,
            jwt_decode_key: None,
            jwt_encoding_key: None,
            jwt_algorithm: None,
        }
    }
    pub fn pool(self, pool: ()) -> Self {
        let mut me = self;
        me.pool = Some(pool);
        me
    }

    pub fn use_api_key(self, key_header_name: HeaderName) -> Self {
        let mut me = self;
        me.api_header_key = Some(key_header_name);
        me
    }

    pub fn api_keys(self, keys: Vec<ApiKey>) -> Self {
        let mut me = self;
        me.loaded_api_keys = Some(keys);
        me
    }

    pub fn jwt_decoding_key(self, key: DecodingKey) -> Self {
        let mut me = self;
        me.jwt_decode_key = Some(key);
        me
    }

    pub fn jwt_encoding_key(self, key: EncodingKey) -> Self {
        let mut me = self;
        me.jwt_encoding_key = Some(key);
        me
    }

    pub fn jwt_algorithm(self, algorithm: Algorithm) -> Self {
        let mut me = self;
        me.jwt_algorithm = Some(algorithm);
        me
    }

    pub fn build(self) -> Result<AuthenticationBackend<ApiKey>, String> {
        if self.pool.is_none() {
            return Err("Missing Pool".to_string());
        }
        if self.jwt_decode_key.is_none() {
            return Err("Missing JWT Decoding key".to_string());
        }
        if self.jwt_encoding_key.is_none() {
            return Err("Missing JWT Encoding key".to_string());
        }

        Ok(AuthenticationBackend {
            sql_pool: self.pool.unwrap(),
            session_store: SessionStore::new(self.loaded_api_keys),
            jwt_authorizer: JwtAuthorizer::new(
                self.jwt_encoding_key.unwrap(),
                self.jwt_decode_key.unwrap(),
                self.jwt_algorithm.unwrap_or(Algorithm::RS256),
            ),
            api_key_header: self.api_header_key,
            authenticated_user: None,
        })
    }
}
