use std::fmt::{self, Display};

use axum::{
    extract::multipart::MultipartError,
    http::StatusCode,
    response::{IntoResponse, Response},
    Json,
};
use error_stack::Context;
use minisign::PError;
use serde::Serialize;
use tracing::error;
use validator::ValidationErrors;
use zip::result::ZipError;

use crate::api::{backend::ApiBackend, routes::users::models::User};

#[derive(Debug)]
pub struct AppError;

impl fmt::Display for AppError {
    fn fmt(&self, fmt: &mut fmt::Formatter<'_>) -> fmt::Result {
        fmt.write_str("Error on executing Application")
    }
}

impl Context for AppError {}

#[derive(Debug, Serialize)]
pub struct ErrorInfo<'a> {
    error_message: &'a str,
    debug_info: Option<&'a String>,
}

#[derive(Debug, PartialEq)]
pub enum ApiError {
    SQLQueryError(String),
    InvalidCredentials,
    InternalError(String),
    AccessDenied,
    MultipartForm(StatusCode, String),
    InvalidRequest(String),
    FileNotFound,
    ValidationError(String),
    ZIPArchive(String),
}

impl ApiError {
    fn as_error_info(&self) -> (StatusCode, ErrorInfo) {
        let (status, error_message, debug_info) = match self {
            Self::InvalidCredentials => (StatusCode::UNAUTHORIZED, "Invalid credentials", None),
            Self::SQLQueryError(error) => (
                StatusCode::INTERNAL_SERVER_ERROR,
                "Error executing SQL request",
                Some(error),
            ),
            Self::InternalError(error) => (
                StatusCode::INTERNAL_SERVER_ERROR,
                "Internal Server Error",
                Some(error),
            ),
            Self::InvalidRequest(s) => (
                StatusCode::BAD_REQUEST,
                "Request contains invalid or missing data",
                Some(s),
            ),
            Self::MultipartForm(c, s) => (*c, "Multipart Error", Some(s)),
            Self::FileNotFound => (StatusCode::NOT_FOUND, "File not found", None),
            Self::ValidationError(s) => (StatusCode::BAD_REQUEST, "Bad request", Some(s)),
            Self::AccessDenied => (StatusCode::FORBIDDEN, "Access Denied", None),
            Self::ZIPArchive(s) => (
                StatusCode::INTERNAL_SERVER_ERROR,
                "Error creating ZIP bundle",
                Some(s),
            ),
        };

        (
            status,
            ErrorInfo {
                error_message,
                debug_info,
            },
        )
    }
}

impl IntoResponse for ApiError {
    fn into_response(self) -> Response {
        let (status, error_info) = self.as_error_info();
        let body = Json(error_info);
        (status, body).into_response()
    }
}

impl From<sqlx::Error> for ApiError {
    fn from(error: sqlx::Error) -> Self {
        error!("{error}");
        Self::SQLQueryError(error.to_string())
    }
}

impl From<axum_jwt_login::Error<User, ApiBackend>> for ApiError {
    fn from(value: axum_jwt_login::Error<User, ApiBackend>) -> Self {
        match value {
            axum_jwt_login::Error::Jwt(error) => ApiError::InternalError(error.to_string()),
            axum_jwt_login::Error::Backend(error) => error,
            _ => unreachable!(),
        }
    }
}

impl From<blake3::HexError> for ApiError {
    fn from(e: blake3::HexError) -> Self {
        Self::InternalError(format!("Could not create Hash from Hex: {e}"))
    }
}

impl From<argon2::Error> for ApiError {
    fn from(_: argon2::Error) -> Self {
        Self::InvalidCredentials
    }
}

impl From<PError> for ApiError {
    fn from(value: PError) -> Self {
        Self::InternalError(format!("MiniSign Error: {value}"))
    }
}

impl From<serde_json::Error> for ApiError {
    fn from(value: serde_json::Error) -> Self {
        Self::InternalError(format!("Error on (de)serialization: {value}"))
    }
}

impl From<MultipartError> for ApiError {
    fn from(value: MultipartError) -> Self {
        Self::MultipartForm(value.status(), value.body_text())
    }
}

impl From<ValidationErrors> for ApiError {
    fn from(value: ValidationErrors) -> Self {
        Self::ValidationError(format!("{value}"))
    }
}

impl From<ZipError> for ApiError {
    fn from(value: ZipError) -> Self {
        Self::ZIPArchive(value.to_string())
    }
}

impl From<std::io::Error> for ApiError {
    fn from(value: std::io::Error) -> Self {
        Self::ZIPArchive(value.to_string())
    }
}

impl Display for ApiError {
    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
        let error_info = self.as_error_info().1;
        f.write_str(&format!(
            "{}: {}",
            error_info.error_message,
            error_info.debug_info.unwrap_or(&"".to_string())
        ))
    }
}

impl std::error::Error for ApiError {}
